#include "xcore.h"

XCore::XCore(const XCsv *csv, const XCsv *src, const int &iterations, const int &seed, QObject *parent) :
		QThread(parent)
{
	_stop = false;
	_iterations = iterations;
	_csv = csv;
	_src = src;

	if (seed)
		srand(seed);
	else
		srand(time(NULL));
}

void XCore::run()
{
	XMatrix srcMatrix;
	XMap srcElements;
	QStringList header;

	foreach (QVariant value, _csv->at(0))
	{
		header << value.toString();
	}

	if (_src != NULL)
	{
		if (_csv->at(0) != _src->at(0))
		{
			emit error(tr("Different headers."));
			return;
		}

		if (!this->readCsv(_src, header, srcMatrix, srcElements))
			return;
	}

	int koffset = srcElements.size();

	XMatrix csvMatrix;
	XMap elements(srcElements);

	if (!this->readCsv(_csv, header, csvMatrix, elements))
		return;

	for (int i = csvMatrix[0].count() - 2; i > 0; --i)
	{
		if (!this->csortMatrix(csvMatrix, i))
			return;
	}

	XMatrix outMatrix;
	XMap outElements;
	XList outRows;
	QList<XResultEntry> results;
	int ke = elements.keys().count();
	int kne = koffset;
	int subsetSize = 0;

	for (int i = 1; i <= 100; ++i)
	{
		long long v1 = kne * 100000;
		long long v2 = (ke * (i * 0.01)) * 100000;

		while (v1 < v2)
		{
			++subsetSize;

			if (!this->iterate(csvMatrix, elements, srcElements, header, outMatrix, outElements, outRows, subsetSize))
				return;

			if (this->_stop)
				return;

			kne = outElements.size();
			v1 = kne * 100000;
		}

		results.append(XResultEntry(kne, outRows));

		emit percentChanged(i);
	}

	emit completed(new XResult(results, _csv, _src));
}


bool XCore::readCsv(const XCsv *csv, const QStringList &header, XMatrix &matrix, XMap &elements) const
{
	int count = 0;

	foreach (XCsvRow row, csv->data())
	{
		if (count == 0)
		{
			++count;
			continue;
		}

		XRow xRow;

		xRow << count++;

		for (int i = 1; i < row.length(); ++i)
		{
			if (row[i] == "*")
			{
				xRow << 0;
			}
			else if (i > 1 && header[i] == header[i - 1] && row[i] == row[i - 1])
			{
				xRow << 0;
			}
			else
			{
				bool ok;
				int value = row[i].toInt(&ok);

				if (!ok)
					value = row[i].toString().toInt(&ok);

				if (!ok || value < 0)
				{
					emit error(tr("Invalid value in csv."));
					return false;
				}

				xRow << value;

				XPair pair(header[i], xRow[i]);

				if (elements.contains(pair))
					++elements[pair];
				else
					elements[pair] = 1;
			}
		}

		matrix << xRow;
	}

	return true;
}

bool XCore::csortMatrix(XMatrix &matrix, const int &column) const
{
	int max = 0;
	int rows = matrix.count();

	foreach (XRow row, matrix)
	{
		int value = row[column];

		if (value > max)
			max = value;
	}

	int *auxArray = new(nothrow) int[max + 1];

	if (!auxArray)
	{
		emit error(tr("Error while allocating memory."));
		return false;
	}

	memset(auxArray, -1, (max + 1) * sizeof(int));

	foreach (XRow row, matrix)
	{
		++auxArray[row[column]];
	}

	for (int i = 1; i <= max; ++i)
	{
		auxArray[i] += auxArray[i - 1] + 1;
	}

	XRow *auxMatrix = new(nothrow) XRow[rows];

	if (!auxMatrix)
	{
		delete[] auxArray;
		emit error(tr("Error while allocating memory."));
		return false;
	}

	for (int i = rows - 1; i >= 0; --i)
	{
		auxMatrix[auxArray[matrix[i][column]]] = matrix[i];
		--auxArray[matrix[i][column]];
	}

	XMatrix output;

	for (int i = 0; i < rows; ++i)
	{
		output << auxMatrix[i];
	}

	matrix = output;

	delete[] auxArray;
	delete[] auxMatrix;
	return true;
}

bool XCore::iterate(const XMatrix &csvMatrix, const XMap &elements, const XMap &srcElements, const QStringList &header, XMatrix &outMatrix, XMap &outElements, QList<int> &outRows, const int &subsetSize) const
{
	int kne = 0;
	int ke = elements.size();
	int rows = csvMatrix.count();
	int *done = new(nothrow) int[rows];

	if (!done)
	{
		emit error(tr("Error while allocating memory."));
		return false;
	}

	for (int it = 0; it < _iterations && kne < ke; ++it)
	{
		XMatrix tempMatrix;
		XMap tempElements = srcElements;
		XList tempRows;

		memset(done, 0, rows * sizeof(int));

		for (int k = 0; k < subsetSize; ++k)
		{
			int x = rand() * ((float)rows / RAND_MAX);

			while (done[x] == 1)
			{
				if (x < (rows - 1))
					++x;
				else
					x = 0;
			}

			done[x] = 1;

			XRow newSet = csvMatrix[x];

			for (int i = 1; i < newSet.length(); ++i)
			{
				if (newSet[i] > 0)
				{
					XPair pair(header[i], newSet[i]);

					if (tempElements.contains(pair))
						++tempElements[pair];
					else
						tempElements[pair] = 1;
				}
			}

			tempMatrix << newSet;
			tempRows << newSet[0];
		}

		bool improving = true;

		while (improving)
		{
			improving = false;

			for (int j = 0; j < tempMatrix.count() - 1; ++j)
			{
				for (int i = 0; i < csvMatrix.count() - 1; ++i)
				{
					int iTester = 0;
					int jTester = 0;

					for (int k = 1; k < tempMatrix[j].count() - 1; ++k)
					{
						if (csvMatrix[i][k] != tempMatrix[j][k] && tempMatrix[j][k] != 0 && csvMatrix[i][k] != 0)
						{
							XPair pair(header[k], csvMatrix[i][k]);

							if (!tempElements.contains(pair))
								++iTester;

							pair = XPair(header[k], tempMatrix[j][k]);

							if(tempElements.contains(pair) && tempElements[pair] == 1)
								++jTester;
						}
					}

					if (iTester > jTester)
					{
						improving = true;

						for (int k = 1; k < tempMatrix[j].count() - 1; ++k)
						{
							if (csvMatrix[i][k] != tempMatrix[j][k] && tempMatrix[j][k] > 0 && csvMatrix[i][k] > 0)
							{
								XPair pair(header[k], csvMatrix[i][k]);

								if (tempElements.contains(pair))
									++tempElements[pair];
								else
									tempElements[pair] = 1;

								pair = XPair(header[k], tempMatrix[j][k]);

								if(tempElements.contains(pair))
								{
									--tempElements[pair];

									if (tempElements[pair] == 0)
										tempElements.remove(pair);
								}
							}
						}

						tempMatrix[j] = csvMatrix[i];
						tempRows[j] =  csvMatrix[i][0];
					}
				}
			}
		}

		if (tempElements.keys().count() > kne)
		{
			kne = tempElements.keys().count();
			outElements = tempElements;
			outMatrix = tempMatrix;
			outRows = tempRows;
		}
	}

	delete[] done;
	return true;
}

void XCore::stopIterating()
{
	this->_stop = true;
}
