#include "xcsveditor.h"

XCsvEditor::XCsvEditor(XCsv *csv, const bool &advancedEditing, QWidget *parent) :
		QDialog(parent)
{
	QPalette palette;

	palette.setColor(QPalette::WindowText, Qt::red);

	_csvModel = new XCsvTableModel(csv, false);
	_csvTable = new XCsvTableWidget(false);
	_errorButton = new QPushButton(tr("Find"));
	_errorLabel = new QLabel(tr("Click on the button to find and correct errors."));

	_csvTable->setModel(_csvModel);
	_errorButton->hide();
	_errorLabel->setPalette(palette);
	_errorLabel->hide();

	this->connect(_csvModel, SIGNAL(modified()), this, SLOT(checkErrors()));
	this->connect(_errorButton, SIGNAL(clicked()), this, SLOT(highlightError()));

	QPushButton *closeButton = new QPushButton(tr("Close"));

	this->connect(closeButton, SIGNAL(clicked()), this, SLOT(close()));

	QHBoxLayout *bottomLayout = new QHBoxLayout;

	bottomLayout->addWidget(_errorButton, 0, Qt::AlignLeft);
	bottomLayout->addWidget(_errorLabel, 1);
	bottomLayout->addWidget(closeButton, 1, Qt::AlignRight);

	QVBoxLayout *mainLayout = new QVBoxLayout;

	if (advancedEditing)
	{
		QToolButton *prependRowButton = new QToolButton;
		QToolButton *appendRowButton = new QToolButton;
		QToolButton *removeRowsButton = new QToolButton;
		QToolButton *prependColumnButton = new QToolButton;
		QToolButton *appendColumnButton = new QToolButton;
		QToolButton *removeColumnsButton = new QToolButton;

		prependRowButton->setIcon(QIcon(":/img/prepend_row"));
		prependRowButton->setIconSize(QSize(33, 33));
		prependRowButton->setToolTip(tr("Insert row before selection"));
		appendRowButton->setIcon(QIcon(":/img/append_row"));
		appendRowButton->setIconSize(QSize(33, 33));
		appendRowButton->setToolTip(tr("Insert row after selection"));
		removeRowsButton->setIcon(QIcon(":/img/remove_row"));
		removeRowsButton->setIconSize(QSize(33, 33));
		removeRowsButton->setToolTip(tr("Remove selected row(s)."));
		prependColumnButton->setIcon(QIcon(":/img/prepend_column"));
		prependColumnButton->setIconSize(QSize(33, 33));
		prependColumnButton->setToolTip(tr("Insert column before selection"));
		appendColumnButton->setIcon(QIcon(":/img/append_column"));
		appendColumnButton->setIconSize(QSize(33, 33));
		appendColumnButton->setToolTip(tr("Insert column after selection"));
		removeColumnsButton->setIcon(QIcon(":/img/remove_column"));
		removeColumnsButton->setIconSize(QSize(33, 33));
		removeColumnsButton->setToolTip(tr("Remove selected column(s)"));

		this->connect(prependRowButton, SIGNAL(clicked()), this, SLOT(prependRow()));
		this->connect(appendRowButton, SIGNAL(clicked()), this, SLOT(appendRow()));
		this->connect(removeRowsButton, SIGNAL(clicked()), this, SLOT(removeRows()));
		this->connect(prependColumnButton, SIGNAL(clicked()), this, SLOT(prependColumn()));
		this->connect(appendColumnButton, SIGNAL(clicked()), this, SLOT(appendColumn()));
		this->connect(removeColumnsButton, SIGNAL(clicked()), this, SLOT(removeColumns()));

		QHBoxLayout *topLayout = new QHBoxLayout;

		topLayout->addWidget(prependRowButton, 0, Qt::AlignLeft);
		topLayout->addWidget(appendRowButton, 0, Qt::AlignLeft);
		topLayout->addWidget(removeRowsButton, 0, Qt::AlignLeft);
		topLayout->addWidget(prependColumnButton, 0, Qt::AlignLeft);
		topLayout->addWidget(appendColumnButton, 0, Qt::AlignLeft);
		topLayout->addWidget(removeColumnsButton, 1, Qt::AlignLeft);

		mainLayout->addLayout(topLayout);
	}

	mainLayout->addWidget(_csvTable);
	mainLayout->addLayout(bottomLayout);

	this->setLayout(mainLayout);
	this->setWindowFlags(Qt::WindowMaximizeButtonHint);
	this->checkErrors();
}

void XCsvEditor::checkErrors()
{
	if (_csvModel->hasErrors())
	{
		_errorButton->show();
		_errorLabel->show();
	}
	else
	{
		_errorButton->hide();
		_errorLabel->hide();
	}
}

void XCsvEditor::prependRow()
{
	QModelIndexList indices = _csvTable->selectionModel()->selection().indexes();

	if (indices.length() == 0)
	{
		_csvModel->insertRow(0);
	}
	else
	{
		int index = _csvModel->rowCount();

		foreach (QModelIndex mi, indices)
		{
			int row = mi.row() + 1;

			if (row < index )
				index = row;
		}

		_csvModel->insertRow(index);
	}
}

void XCsvEditor::appendRow()
{
	QModelIndexList indices = _csvTable->selectionModel()->selection().indexes();

	if (indices.length() == 0)
	{
		_csvModel->insertRow(_csvModel->rowCount() + 1);
	}
	else
	{
		int index = 0;

		foreach (QModelIndex mi, indices)
		{
			int row = mi.row() + 2;

			if (row > index )
				index = row;
		}

		_csvModel->insertRow(index);
	}
}

void XCsvEditor::removeRows()
{
	QModelIndexList indices = _csvTable->selectionModel()->selection().indexes();
	QList<int> rows;

	foreach (QModelIndex mi, indices)
	{
		if (!rows.contains(mi.row()))
			rows.append(mi.row());
	}

	qSort(rows.begin(), rows.end(), qGreater<int>());

	foreach (int row, rows)
	{
		_csvModel->removeRow(row);
	}
}

void XCsvEditor::prependColumn()
{
	QModelIndexList indices = _csvTable->selectionModel()->selection().indexes();

	if (indices.length() == 0)
	{
		_csvModel->insertColumn(0);
	}
	else
	{
		int index = _csvModel->columnCount();

		foreach (QModelIndex mi, indices)
		{
			int column = mi.column() + 1;

			if (column < index )
				index = column;
		}

		_csvModel->insertColumn(index);
	}
}

void XCsvEditor::appendColumn()
{
	QModelIndexList indices = _csvTable->selectionModel()->selection().indexes();

	if (indices.length() == 0)
	{
		_csvModel->insertColumn(_csvModel->columnCount() + 1);
	}
	else
	{
		int index = 0;

		foreach (QModelIndex mi, indices)
		{
			int column = mi.column() + 2;

			if (column > index )
				index = column;
		}

		_csvModel->insertColumn(index);
	}
}

void XCsvEditor::removeColumns()
{
	QModelIndexList indices = _csvTable->selectionModel()->selection().indexes();
	QList<int> columns;

	foreach (QModelIndex mi, indices)
	{
		if (!columns.contains(mi.column()))
			columns.append(mi.column());
	}

	qSort(columns.begin(), columns.end(), qGreater<int>());

	foreach (int column, columns)
	{
		_csvModel->removeColumn(column);
	}
}

void XCsvEditor::highlightError()
{
	if (!_csvModel->hasErrors())
		return;

	XCsvError error = _csvModel->errors().first();

	_csvTable->setCurrentIndex(_csvModel->index(error.first, error.second));
	_csvTable->setFocus();
}
