#include "xcsvtablemodel.h"

XCsvTableModel::XCsvTableModel(XCsv *csv, const bool &readOnly, QObject *parent) :
		QAbstractTableModel(parent)
{
	_csv = csv;
	_readOnly = readOnly;

	this->check();
}

const QList<XCsvError> &XCsvTableModel::errors() const
{
	return _errors;
}

void XCsvTableModel::check()
{
	for (int i = 0; i < _csv->rows() - 1; ++i)
	{
		for (int j = 0; j < _csv->columns() - 1; ++j)
		{
			this->checkCell(i, j);
			QVariant value = _csv->at(i, j);
		}
	}
}

bool XCsvTableModel::isError(const QVariant &value) const
{
	if (value.isNull() || !value.isValid())
		return true;

	if (value.toString() == "*")
		return false;

	bool ok;
	int num = value.toInt(&ok);

	return !ok || num < 0;
}

bool XCsvTableModel::hasErrors() const
{
	return _errors.length() > 0;
}

int XCsvTableModel::rowCount(const QModelIndex &parent) const
{
	return _csv->rows() - 1;
}

int XCsvTableModel::columnCount(const QModelIndex &parent) const
{
	return _csv->columns() - 1;
}

QVariant XCsvTableModel::headerData(int section, Qt::Orientation orientation, int role) const
{
	switch (role)
	{
	case Qt::DisplayRole:
		switch (orientation)
		{
		case Qt::Horizontal:
			return _csv->at(0, section + 1);
		case Qt::Vertical:
			return _csv->at(section + 1, 0);
		}
	}

	return QVariant::Invalid;
}

QVariant XCsvTableModel::data(const QModelIndex &index, int role) const
{
	switch (role)
	{
	case Qt::BackgroundRole:
		{
			QVariant value = _csv->at(index.row() + 1, index.column() + 1);

			if (value.toString() == "*")
				return QBrush(Qt::lightGray);
			else if (this->isError(value))
				return QBrush(Qt::red);

			break;
		}
	case Qt::DisplayRole:
		return _csv->at(index.row() + 1, index.column() + 1);
	case Qt::TextAlignmentRole:
		return Qt::AlignCenter;
	}

	return QVariant::Invalid;
}

Qt::ItemFlags XCsvTableModel::flags(const QModelIndex &index) const
{
	Qt::ItemFlags flags = Qt::ItemIsEnabled | Qt::ItemIsSelectable;

	if (!_readOnly)
		flags |= Qt::ItemIsEditable;

	return flags;
}

bool XCsvTableModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
	if (_readOnly)
		return false;

	bool result = false;
	int row = index.row();
	int column = index.column();

	switch (role)
	{
	case Qt::EditRole:
		result = _csv->setCell(row + 1, column + 1, value);
	}

	if (result)
	{
		this->checkCell(row, column);
		emit dataChanged(index, index);
	}

	return result;
}

bool XCsvTableModel::setHeaderData(int section, Qt::Orientation orientation, const QVariant &value, int role)
{
	if (_readOnly)
		return false;

	bool result = false;

	switch (role)
	{
	case Qt::EditRole:
		switch (orientation)
		{
		case Qt::Horizontal:
			result = _csv->setCell(0, section + 1, value);
			break;
		case Qt::Vertical:
			result = _csv->setCell(section + 1, 0, value);
			break;
		}
	}

	if (result)
		emit headerDataChanged(orientation, section, section);

	return result;
}

bool XCsvTableModel::insertRows(int row, int count, const QModelIndex &parent)
{
	if (count != 1)
		return false;

	if (row == 0)
		row = 1;

	this->beginInsertRows(parent, row - 1, row - 1);

	bool result = _csv->addRow(row, tr("Row"), "*");

	this->endInsertRows();

	if (result)
	{
		for (int i = 0; i < _errors.length(); ++i)
		{
			if (_errors[i].first >= row - 1)
				++_errors[i].first;
		}

		emit modified();
	}

	return result;
}

bool XCsvTableModel::removeRows(int row, int count, const QModelIndex &parent)
{
	if (count != 1 || _csv->rows() < 3)
		return false;

	this->beginRemoveRows(parent, row, row);

	bool result = _csv->removeRow(row + 1);

	this->endRemoveRows();

	if (result)
	{
		QList<XCsvError> errors;

		for (int i = 0; i < _errors.length(); ++i)
		{
			if (_errors[i].first == row)
				errors << _errors[i];
			else if (_errors[i].first > row)
				--_errors[i].first;
		}

		foreach (XCsvError error, errors)
		{
			_errors.removeOne(error);
		}

		emit modified();
	}

	return result;
}

bool XCsvTableModel::insertColumns(int column, int count, const QModelIndex &parent)
{
	if (count != 1)
		return false;

	if (column == 0)
		column = 1;

	this->beginInsertColumns(parent, column - 1, column - 1);

	bool result = _csv->addColumn(column, tr("Column"), "*");

	this->endInsertColumns();

	if (result)
	{
		for (int i = 0; i < _errors.length(); ++i)
		{
			if (_errors[i].second >= column - 1)
				++_errors[i].second;
		}

		emit modified();
	}

	return result;
}

bool XCsvTableModel::removeColumns(int column, int count, const QModelIndex &parent)
{
	if (count != 1 || _csv->columns() < 3)
		return false;

	this->beginRemoveColumns(parent, column, column);

	bool result = _csv->removeColumn(column + 1);

	this->endRemoveColumns();

	if (result)
	{
		QList<XCsvError> errors;

		for (int i = 0; i < _errors.length(); ++i)
		{
			if (_errors[i].second == column)
				errors << _errors[i];
			else if (_errors[i].second > column)
			{
				--_errors[i].second;
			}
		}

		foreach (XCsvError error, errors)
		{
			_errors.removeOne(error);
		}

		emit modified();
	}

	return result;
}

void XCsvTableModel::checkCell(const int &row, const int &column)
{
	QVariant value = _csv->at(row + 1, column + 1);
	XCsvError error(row, column);

	if (isError(value))
	{
		if (!_errors.contains(error))
			_errors.append(error);
	}
	else if (_errors.contains(error))
	{
		_errors.removeAt(_errors.indexOf(error));
	}

	emit modified();
}
