#include "xwizard.h"

XWizard::XWizard(QWidget *parent) :
		QWizard(parent)
{
	this->setPage(PageFirst, new XFirstPage);
	this->setPage(PageCsvSelection, new XCsvSelectionPage);
	this->setPage(PageManualEditing, new XManualEditingPage);
	this->setPage(PageSettings, new XSettingsPage);
	this->connect(this, SIGNAL(finished(int)), this, SLOT(finished(int)));
}

void XWizard::finished(int result)
{
	XCsvSelectionPage *filePage = (XCsvSelectionPage *)this->page(PageCsvSelection);
	XManualEditingPage *manualPage = (XManualEditingPage *)this->page(PageManualEditing);
	XCsv *fileCsv = filePage->csv();
	XCsv *fileSrc = filePage->src();
	XCsv *manualCsv = manualPage->csv();

	if (result == QWizard::Accepted)
	{
		if (field("file").toBool())
		{
			delete manualCsv;

			emit finished(fileCsv, fileSrc, this->field("iterations").toInt(), this->field("seed").toInt());
		}
		else
		{
			delete fileCsv;
			delete fileSrc;

			emit finished(manualCsv, NULL, this->field("iterations").toInt(), this->field("seed").toInt());
		}
	}
	else
	{
		delete fileCsv;
		delete fileSrc;
		delete manualCsv;
	}
}

XFirstPage::XFirstPage(QWidget *parent)
	: QWizardPage(parent)
{
	_fileInputRadioButton = new QRadioButton(tr("Input from an existing file"));
	_fileInputRadioButton->setChecked(true);
	_manualInputRadioButton = new QRadioButton(tr("Input data manually"));

	this->registerField("file", _fileInputRadioButton);
	this->registerField("manual", _manualInputRadioButton);

	QVBoxLayout *layout = new QVBoxLayout;

	layout->addWidget(_fileInputRadioButton);
	layout->addWidget(_manualInputRadioButton);

	this->setLayout(layout);
	this->setTitle(tr("Start"));
	this->setSubTitle(tr("Select the action you want to perform"));
}

int XFirstPage::nextId() const
{
	if (_fileInputRadioButton->isChecked())
		return XWizard::PageCsvSelection;
	else if (_manualInputRadioButton->isChecked())
		return XWizard::PageManualEditing;
	else
		return XWizard::PageFirst;
}

XCsvSelectionPage::XCsvSelectionPage(QWidget *parent)
	: QWizardPage(parent)
{
	_csv = NULL;
	_csvModel = NULL;
	_csvPathLineEdit = new QLineEdit;
	_csvPathLineEdit->setReadOnly(true);
	_csvBrowseButton = new QPushButton(tr("Browse..."));
	_csvErrorLabel = new QLabel;
	_csvEditButton = new QPushButton(tr("Edit"));
	_src = NULL;
	_srcModel = NULL;
	_srcCheckBox = new QCheckBox(tr("Start from an existing core collection"));
	_srcPathLineEdit = new QLineEdit;
	_srcPathLineEdit->setReadOnly(true);
	_srcBrowseButton = new QPushButton(tr("Browse..."));
	_srcErrorLabel = new QLabel;
	_srcEditButton = new QPushButton(tr("Edit"));

	this->connect(_csvBrowseButton, SIGNAL(clicked()), this, SLOT(csvBrowseButtonClicked()));
	this->connect(_csvEditButton, SIGNAL(clicked()), this, SLOT(csvEditButtonClicked()));
	this->connect(_srcCheckBox, SIGNAL(toggled(bool)), this, SLOT(srcCheckboxToggled(bool)));
	this->connect(_srcBrowseButton, SIGNAL(clicked()), this, SLOT(srcBrowseButtonClicked()));
	this->connect(_srcEditButton, SIGNAL(clicked()), this, SLOT(srcEditButtonClicked()));

	QHBoxLayout *csvFileLayout = new QHBoxLayout;

	csvFileLayout->addWidget(_csvPathLineEdit);
	csvFileLayout->addWidget(_csvBrowseButton);

	QHBoxLayout *csvErrorLayout = new QHBoxLayout;

	csvErrorLayout->addWidget(_csvErrorLabel, 0, Qt::AlignLeft);
	csvErrorLayout->addWidget(_csvEditButton, 0, Qt::AlignRight);

	QHBoxLayout *srcFileLayout = new QHBoxLayout;

	srcFileLayout->addWidget(_srcPathLineEdit);
	srcFileLayout->addWidget(_srcBrowseButton);

	QHBoxLayout *srcErrorLayout = new QHBoxLayout;

	srcErrorLayout->addWidget(_srcErrorLabel, 0, Qt::AlignLeft);
	srcErrorLayout->addWidget(_srcEditButton, 0, Qt::AlignRight);

	QVBoxLayout *layout = new QVBoxLayout;

	layout->addLayout(csvFileLayout);
	layout->addLayout(csvErrorLayout);
	layout->addWidget(_srcCheckBox);
	layout->addLayout(srcFileLayout);
	layout->addLayout(srcErrorLayout);

	setLayout(layout);
	setTitle(tr("Select file"));
	setSubTitle(tr("Preview and edit the input spreadsheet"));
}

XCsv *XCsvSelectionPage::csv() const
{
	return _csv;
}

XCsv *XCsvSelectionPage::src() const
{
	return _src;
}

void XCsvSelectionPage::initializePage()
{
	delete _csv;
	delete _csvModel;
	delete _src;
	delete _srcModel;

	_csv = NULL;
	_csvModel = NULL;
	_src = NULL;
	_srcModel = NULL;

	_csvPathLineEdit->setText("");
	_srcCheckBox->setChecked(false);
	_csvErrorLabel->setVisible(false);
	_csvEditButton->setVisible(false);
	_srcCheckBox->setVisible(false);

	this->switchSrcSection(false);
}

bool XCsvSelectionPage::isComplete() const
{
	bool complete = _csv != NULL && _csvModel != NULL && !_csvModel->hasErrors();

	if (_srcCheckBox->isChecked())
		complete = complete && _src != NULL && _srcModel != NULL && !_srcModel->hasErrors();

	return complete;
}

int XCsvSelectionPage::nextId() const
{
	return XWizard::PageSettings;
}

void XCsvSelectionPage::switchState(QLabel *label, QPushButton *button, const bool &errors)
{
	QPalette palette;
	QColor color;
	QString text;

	if (errors)
	{
		palette.setColor(QPalette::Foreground, Qt::red);
		text = tr("There are errors in the csv file. Click on the Edit button to fix them.");
	}
	else
	{
		text = tr("Click on the Edit button to make changes to the csv file.");
	}

	label->setPalette(palette);
	label->setText(text);
	label->setVisible(true);
	button->setVisible(true);

	this->completeChanged();
}

void XCsvSelectionPage::switchSrcSection(const bool &enable)
{
	delete _src;
	delete _srcModel;

	_src = NULL;
	_srcModel = NULL;

	_srcPathLineEdit->setText("");
	_srcErrorLabel->setVisible(false);
	_srcEditButton->setVisible(false);

	if (enable)
	{
		_srcPathLineEdit->setVisible(true);
		_srcBrowseButton->setVisible(true);
	}
	else
	{
		_srcPathLineEdit->setVisible(false);
		_srcBrowseButton->setVisible(false);
	}
}

XCsv *XCsvSelectionPage::parseFile(const QString &path) const
{
	if (path.endsWith(".xls"))
	{
		BasicExcel xls(path.toLatin1().data());
		BasicExcelWorksheet *sheet = xls.GetWorksheet((size_t)0);

		int rows = sheet->GetTotalRows();
		int cols = sheet->GetTotalCols();

		XCsvData data;

		for (int i = 0; i < rows; ++i)
		{
			XCsvRow row;
			for (int j = 0; j < cols; ++j)
			{
				BasicExcelCell *cell = sheet->Cell(i, j);
				const char *str = cell->GetString();
				QVariant value;

				if (!str)
				{
					bool ok;
					int integer = cell->GetInteger(&ok);

					if (!ok)
					{
						QMessageBox::critical(NULL, tr("Error"), tr("Error while parsing the .xls file: cell at row %1 and column %2 does not appear to be a valid string or number.").arg(i).arg(j));
						return NULL;
					}

					value = integer;
				}
				else
				{
					value = QString(str);
				}

				row << value;
			}

			data << row;
		}

		return new XCsv(data);
	}
	else if(path.endsWith(".csv"))
	{
		return new XCsv(path);
	}
	else
	{
		QMessageBox::critical(NULL, tr("Error"), tr("The file you selected is not a .csv or a .xls file."));
		return NULL;
	}
}

void XCsvSelectionPage::csvBrowseButtonClicked()
{
	QString path = QFileDialog::getOpenFileName(this, "", "", SPREADSHEET_FILTER).trimmed();

	if (path.isEmpty())
		return;

	XCsv *csv = this->parseFile(path);

	if (!csv)
		return;

	delete _csv;
	delete _csvModel;

	_csv = csv;
	_csvModel = new XCsvTableModel(csv, true, this);
	_csvPathLineEdit->setText(path);
	_srcCheckBox->setVisible(true);
	_srcCheckBox->setChecked(false);

	this->switchState(_csvErrorLabel, _csvEditButton, _csvModel->hasErrors());
}

void XCsvSelectionPage::csvEditButtonClicked()
{
	if (!_csv)
		return;

	XCsvEditor editor(_csv, true);

	centerResize(editor, 60);
	editor.exec();
	_csvModel->check();
	this->switchState(_csvErrorLabel, _csvEditButton, _csvModel->hasErrors());
}

void XCsvSelectionPage::srcCheckboxToggled(bool checked)
{
	this->switchSrcSection(checked);
	this->completeChanged();
}

void XCsvSelectionPage::srcBrowseButtonClicked()
{
	QString path = QFileDialog::getOpenFileName(this, "", "", SPREADSHEET_FILTER).trimmed();

	if (path.isEmpty())
		return;

	XCsv *src = this->parseFile(path);

	if (!src)
		return;

	if (_src)
		delete _src;

	if (_srcModel)
		delete _srcModel;

	_src = src;
	_srcModel = new XCsvTableModel(src, true, this);
	_srcPathLineEdit->setText(path);

	this->switchState(_srcErrorLabel, _srcEditButton, _srcModel->hasErrors());
}

void XCsvSelectionPage::srcEditButtonClicked()
{
	if (!_src)
		return;

	XCsvEditor editor(_src, true);

	centerResize(editor, 60);
	editor.exec();
	_srcModel->check();
	this->switchState(_srcErrorLabel, _srcEditButton, _srcModel->hasErrors());
}

XManualEditingPage::XManualEditingPage(QWidget *parent)
	: QWizardPage(parent)
{
	_csv = NULL;
	_csvModel = NULL;
	_rowsLabel = new QLabel(tr("Rows"));
	_rowsSpinBox = new QSpinBox;
	_rowsSpinBox->setMinimum(1);
	_columnsLabel = new QLabel(tr("Columns"));
	_columnsSpinBox = new QSpinBox;
	_columnsSpinBox->setMinimum(1);
	_fillLabel = new QLabel(tr("Fill character"));
	_fillLineEdit = new QLineEdit;
	_createLabel = new QLabel(tr("Click on the button to create a table using the values above."));
	_createButton = new QPushButton(tr("Create"));
	_editLabel = new QLabel;
	_editButton = new QPushButton(tr("Edit"));

	this->connect(_createButton, SIGNAL(clicked()), this, SLOT(createButtonClicked()));
	this->connect(_editButton, SIGNAL(clicked()), this, SLOT(editButtonClicked()));

	QGridLayout *gridLayout = new QGridLayout;

	gridLayout->addWidget(_rowsLabel, 0, 0);
	gridLayout->addWidget(_rowsSpinBox, 0, 1);
	gridLayout->addWidget(_columnsLabel, 1, 0);
	gridLayout->addWidget(_columnsSpinBox, 1, 1);
	gridLayout->addWidget(_fillLabel, 2, 0);
	gridLayout->addWidget(_fillLineEdit, 2, 1);

	QHBoxLayout *createLayout = new QHBoxLayout;

	createLayout->addWidget(_createLabel, 0, Qt::AlignLeft);
	createLayout->addWidget(_createButton, 0, Qt::AlignRight);

	QHBoxLayout *editLayout = new QHBoxLayout;

	editLayout->addWidget(_editLabel, 0, Qt::AlignLeft);
	editLayout->addWidget(_editButton, 0, Qt::AlignRight);

	QVBoxLayout *layout = new QVBoxLayout;

	layout->addLayout(gridLayout);
	layout->addLayout(createLayout);
	layout->addLayout(editLayout);

	this->setLayout(layout);
	this->setTitle(tr("Manual editing"));
	this->setSubTitle(tr("Create and edit a new table"));
}

XCsv *XManualEditingPage::csv() const
{
	return _csv;
}

void XManualEditingPage::initializePage()
{
	delete _csv;
	delete _csvModel;

	_csv = NULL;
	_csvModel = NULL;

	_rowsSpinBox->setValue(10);
	_columnsSpinBox->setValue(10);
	_fillLineEdit->setText("*");
	_createLabel->setVisible(true);
	_createButton->setVisible(true);
	_editLabel->setVisible(false);
	_editButton->setVisible(false);
}

bool XManualEditingPage::isComplete() const
{
	return _csv != NULL && _csvModel != NULL && !_csvModel->hasErrors();
}

int XManualEditingPage::nextId() const
{
	return XWizard::PageSettings;
}

void XManualEditingPage::switchState(const bool &errors)
{
	QPalette palette;
	QColor color;
	QString text;

	if (errors)
	{
		palette.setColor(QPalette::Foreground, Qt::red);
		text = tr("There are errors in the csv file. Click on the Edit button to fix them.");
	}
	else
	{
		text = tr("Click on the Edit button to make changes to the csv file.");
	}

	_editLabel->setPalette(palette);
	_editLabel->setText(text);
	_editLabel->setVisible(true);
	_editButton->setVisible(true);

	this->completeChanged();
}

void XManualEditingPage::createButtonClicked()
{
	QString fill =_fillLineEdit->text();
	XCsvData data;

	for (int i = 0; i <= _rowsSpinBox->value(); ++i)
	{
		XCsvRow row;

		for (int j = 0; j <= _columnsSpinBox->value(); ++j)
		{
			if (i == 0 && j == 0)
				row << "Name";
			else if (i == 0)
				row << "Column " + QString::number(j);
			else if (j == 0)
				row << "Row " + QString::number(i);
			else
				row << fill;
		}

		data << row;
	}

	XCsv *csv = new XCsv(data);

	delete _csv;
	delete _csvModel;

	_csv = csv;
	_csvModel = new XCsvTableModel(csv, true, this);

	_createLabel->setVisible(false);
	_createButton->setVisible(false);

	this->switchState(_csvModel->hasErrors());
	this->editButtonClicked();
}

void XManualEditingPage::editButtonClicked()
{
	if (!_csv)
		return;

	XCsvEditor editor(_csv, true);

	centerResize(editor, 60);
	editor.exec();
	_csvModel->check();
	this->switchState(_csvModel->hasErrors());
}

XSettingsPage::XSettingsPage(QWidget *parent)
	: QWizardPage(parent)
{
	_iterationsLabel = new QLabel(tr("Iterations"));
	_iterationsSpinBox = new QSpinBox;
	_seedLabel = new QLabel(tr("Rand seed (0 = autogenerated)"));
	_seedSpinBox = new QSpinBox;
	_seedSpinBox->setMaximum(INT_MAX - 1);

	this->registerField("iterations*", _iterationsSpinBox);
	this->registerField("seed", _seedSpinBox);

	QGridLayout *layout = new QGridLayout;

	layout->addWidget(_iterationsLabel, 0, 0);
	layout->addWidget(_iterationsSpinBox, 0, 1);
	layout->addWidget(_seedLabel, 1, 0);
	layout->addWidget(_seedSpinBox, 1, 1);

	this->setLayout(layout);
	this->setTitle(tr("Algorithm Settings"));
	this->setSubTitle(tr("Adjust the algorithm's parameters"));
}

void XSettingsPage::initializePage()
{
	_iterationsSpinBox->setValue(1);
}

int XSettingsPage::nextId() const
{
	return -1;
}
